@@ -13,7 +13,9 @@ use Carp;
 
 use Tangence::Constants;
 
-our $VERSION = '0.18';
+require Tangence::Type;
+
+our $VERSION = '0.19';
 
 sub build_accessor
 {
@@ -208,6 +210,12 @@ sub _accessor_for_objset
    };
 }
 
+sub make_type
+{
+   shift;
+   return Tangence::Type->new( @_ );
+}
+
 package # hide from CPAN
    Tangence::Property::_Iterator;
 
