@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Tangence::Object );
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 use Carp;
 
@@ -17,6 +17,7 @@ use Tangence::Constants;
 use Tangence::Class;
 use Tangence::Property;
 use Tangence::Struct;
+use Tangence::Type;
 
 use Tangence::Compiler::Parser;
 
@@ -206,6 +207,12 @@ sub make_property
    return Tangence::Property->new( @_ );
 }
 
+sub make_type
+{
+   my $self = shift;
+   return Tangence::Type->new( @_ );
+}
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
